@@ -1,6 +1,7 @@
 from alphafold3_pytorch.attention import (
     Attention,
-    Attend
+    Attend,
+    full_pairwise_repr_to_windowed
 )
 
 from alphafold3_pytorch.alphafold3 import (
@@ -33,7 +34,8 @@ from alphafold3_pytorch.alphafold3 import (
 
 from alphafold3_pytorch.trainer import (
     Trainer,
-    Alphafold3Input
+    DataLoader,
+    AtomInput
 )
 
 __all__ = [
@@ -64,6 +66,6 @@ __all__ = [
     ConfidenceHead,
     DistogramHead,
     Alphafold3,
-    Alphafold3Input,
+    AtomInput,
     Trainer
 ]
