@@ -26,9 +26,19 @@ from alphafold3_pytorch import (
 
 from alphafold3_pytorch.alphafold3 import (
     mean_pool_with_lens,
-    repeat_consecutive_with_lens
+    repeat_consecutive_with_lens,
+    full_pairwise_repr_to_windowed,
+    atom_ref_pos_to_atompair_inputs
 )
 
+def test_atom_ref_pos_to_atompair_inputs():
+    atom_ref_pos = torch.randn(16, 3)
+    atom_ref_space_uid = torch.ones(16).long()
+
+    atompair_inputs = atom_ref_pos_to_atompair_inputs(atom_ref_pos, atom_ref_space_uid)
+
+    assert atompair_inputs.shape == (16, 16, 5)
+
 def test_mean_pool_with_lens():
     seq = torch.tensor([[[1.], [1.], [1.], [2.], [2.], [2.], [2.], [1.], [1.]]])
     lens = torch.tensor([[3, 4, 2]]).long()
@@ -65,6 +75,14 @@ def test_weighted_rigid_align():
     rmsd = torch.sqrt(((pred_coords - aligned_coords) ** 2).sum(dim=-1).mean(dim=-1))
     assert (rmsd < 1e-5).all()
 
+    random_augment_fn = CentreRandomAugmentation()
+    aligned_coords = align_fn(random_augment_fn(pred_coords), pred_coords, weights)
+
+    # `pred_coords` should match a random augmentation of itself after alignment
+
+    rmsd = torch.sqrt(((pred_coords - aligned_coords) ** 2).sum(dim=-1).mean(dim=-1))
+    assert (rmsd < 1e-5).all()
+
 def test_weighted_rigid_align_with_mask():
     pred_coords = torch.randn(2, 100, 3)
     true_coords = torch.randn(2, 100, 3)
@@ -119,14 +137,15 @@ def test_express_coordinates_in_frame():
 
 def test_compute_alignment_error():
     pred_coords = torch.randn(2, 100, 3)
-    true_coords = torch.randn(2, 100, 3)
     pred_frames = torch.randn(2, 100, 3, 3)
-    true_frames = torch.randn(2, 100, 3, 3)
+
+    # `pred_coords` should match itself in frame basis
 
     error_fn = ComputeAlignmentError()
-    alignment_errors = error_fn(pred_coords, true_coords, pred_frames, true_frames)
+    alignment_errors = error_fn(pred_coords, pred_coords, pred_frames, pred_frames)
 
     assert alignment_errors.shape == (2, 100)
+    assert (alignment_errors.mean(-1) < 1e-3).all()
 
 def test_centre_random_augmentation():
     coords = torch.randn(2, 100, 3)
@@ -137,7 +156,10 @@ def test_centre_random_augmentation():
     assert augmented_coords.shape == coords.shape
 
 
-def test_pairformer():
+@pytest.mark.parametrize('recurrent_depth', (1, 2))
+def test_pairformer(
+    recurrent_depth
+):
     single = torch.randn(2, 16, 384)
     pairwise = torch.randn(2, 16, 16, 128)
     mask = torch.randint(0, 2, (2, 16)).bool()
@@ -145,6 +167,7 @@ def test_pairformer():
     pairformer = PairformerStack(
         depth = 4,
         num_register_tokens = 4,
+        recurrent_depth = recurrent_depth
     )
 
     single_out, pairwise_out = pairformer(
@@ -176,8 +199,8 @@ def test_msa_module():
 
     assert pairwise.shape == pairwise_out.shape
 
-
-def test_diffusion_transformer():
+@pytest.mark.parametrize('use_linear_attn', (False, True))
+def test_diffusion_transformer(use_linear_attn):
 
     single = torch.randn(2, 16, 384)
     pairwise = torch.randn(2, 16, 16, 128)
@@ -185,7 +208,8 @@ def test_diffusion_transformer():
 
     diffusion_transformer = DiffusionTransformer(
         depth = 2,
-        heads = 16
+        heads = 16,
+        use_linear_attn = use_linear_attn
     )
 
     single_out = diffusion_transformer(
@@ -214,7 +238,9 @@ def test_sequence_local_attn():
 def test_diffusion_module():
 
     seq_len = 16
-    atom_seq_len = 27 * 16
+
+    molecule_atom_lens = torch.randint(1, 3, (2, seq_len))
+    atom_seq_len = molecule_atom_lens.sum(dim = -1).amax()
 
     noised_atom_pos = torch.randn(2, atom_seq_len, 3)
     atom_feats = torch.randn(2, atom_seq_len, 128)
@@ -251,7 +277,8 @@ def test_diffusion_module():
         single_trunk_repr = single_trunk_repr,
         single_inputs_repr = single_inputs_repr,
         pairwise_trunk = pairwise_trunk,
-        pairwise_rel_pos_feats = pairwise_rel_pos_feats
+        pairwise_rel_pos_feats = pairwise_rel_pos_feats,
+        molecule_atom_lens = molecule_atom_lens
     )
 
     assert noised_atom_pos.shape == atom_pos_update.shape
@@ -271,6 +298,7 @@ def test_diffusion_module():
         single_inputs_repr = single_inputs_repr,
         pairwise_trunk = pairwise_trunk,
         pairwise_rel_pos_feats = pairwise_rel_pos_feats,
+        molecule_atom_lens = molecule_atom_lens,
         add_bond_loss = True
     )
 
@@ -284,18 +312,19 @@ def test_diffusion_module():
         single_trunk_repr = single_trunk_repr,
         single_inputs_repr = single_inputs_repr,
         pairwise_trunk = pairwise_trunk,
-        pairwise_rel_pos_feats = pairwise_rel_pos_feats
+        pairwise_rel_pos_feats = pairwise_rel_pos_feats,
+        molecule_atom_lens = molecule_atom_lens
     )
 
     assert sampled_atom_pos.shape == noised_atom_pos.shape
     
 def test_relative_position_encoding():
-    additional_residue_feats = torch.randn(8, 100, 10)
+    additional_molecule_feats = torch.randn(8, 100, 10)
 
     embedder = RelativePositionEncoding()
 
     rpe_embed = embedder(
-        additional_residue_feats = additional_residue_feats
+        additional_molecule_feats = additional_molecule_feats
     )
 
 def test_template_embed():
@@ -341,11 +370,13 @@ def test_confidence_head():
 
 def test_input_embedder():
 
-    atom_seq_len = 16 * 27
+    molecule_atom_lens = torch.randint(0, 3, (2, 16))
+    atom_seq_len = molecule_atom_lens.sum(dim = -1).amax()
     atom_inputs = torch.randn(2, atom_seq_len, 77)
+    atompair_inputs = torch.randn(2, atom_seq_len, atom_seq_len, 5)
+
     atom_mask = torch.ones((2, atom_seq_len)).bool()
-    atompair_feats = torch.randn(2, atom_seq_len, atom_seq_len, 16)
-    additional_residue_feats = torch.randn(2, 16, 10)
+    additional_molecule_feats = torch.randn(2, 16, 10)
 
     embedder = InputFeatureEmbedder(
         dim_atom_inputs = 77,
@@ -354,8 +385,9 @@ def test_input_embedder():
     embedder(
         atom_inputs = atom_inputs,
         atom_mask = atom_mask,
-        atompair_feats = atompair_feats,
-        additional_residue_feats = additional_residue_feats
+        atompair_inputs = atompair_inputs,
+        molecule_atom_lens = molecule_atom_lens,
+        additional_molecule_feats = additional_molecule_feats
     )
 
 def test_distogram_head():
@@ -365,17 +397,26 @@ def test_distogram_head():
 
     logits = distogram_head(pairwise_repr)
 
-
-def test_alphafold3():
+@pytest.mark.parametrize('window_atompair_inputs', (True, False))
+def test_alphafold3(
+    window_atompair_inputs: bool
+):
     seq_len = 16
-    atom_seq_len = seq_len * 27
+    atoms_per_window = 27
+
+    molecule_atom_lens = torch.randint(1, 3, (2, seq_len))
+    atom_seq_len = molecule_atom_lens.sum(dim = -1).amax()
 
     token_bond = torch.randint(0, 2, (2, seq_len, seq_len)).bool()
 
     atom_inputs = torch.randn(2, atom_seq_len, 77)
-    atom_lens = torch.randint(0, 27, (2, seq_len))
-    atompair_feats = torch.randn(2, atom_seq_len, atom_seq_len, 16)
-    additional_residue_feats = torch.randn(2, seq_len, 10)
+
+    atompair_inputs = torch.randn(2, atom_seq_len, atom_seq_len, 5)
+
+    if window_atompair_inputs:
+        atompair_inputs = full_pairwise_repr_to_windowed(atompair_inputs, window_size = atoms_per_window)
+
+    additional_molecule_feats = torch.randn(2, seq_len, 10)
 
     template_feats = torch.randn(2, 2, seq_len, seq_len, 44)
     template_mask = torch.ones((2, 2)).bool()
@@ -384,7 +425,7 @@ def test_alphafold3():
     msa_mask = torch.ones((2, 7)).bool()
 
     atom_pos = torch.randn(2, atom_seq_len, 3)
-    residue_atom_indices = torch.randint(0, 27, (2, seq_len))
+    molecule_atom_indices = molecule_atom_lens - 1
 
     pae_labels = torch.randint(0, 64, (2, seq_len, seq_len))
     pde_labels = torch.randint(0, 64, (2, seq_len, seq_len))
@@ -393,6 +434,7 @@ def test_alphafold3():
 
     alphafold3 = Alphafold3(
         dim_atom_inputs = 77,
+        atoms_per_window = atoms_per_window,
         dim_template_feats = 44,
         num_dist_bins = 38,
         confidence_head_kwargs = dict(
@@ -417,16 +459,16 @@ def test_alphafold3():
     loss, breakdown = alphafold3(
         num_recycling_steps = 2,
         atom_inputs = atom_inputs,
-        residue_atom_lens = atom_lens,
-        atompair_feats = atompair_feats,
-        additional_residue_feats = additional_residue_feats,
+        molecule_atom_lens = molecule_atom_lens,
+        atompair_inputs = atompair_inputs,
+        additional_molecule_feats = additional_molecule_feats,
         token_bond = token_bond,
         msa = msa,
         msa_mask = msa_mask,
         templates = template_feats,
         template_mask = template_mask,
         atom_pos = atom_pos,
-        residue_atom_indices = residue_atom_indices,
+        molecule_atom_indices = molecule_atom_indices,
         pae_labels = pae_labels,
         pde_labels = pde_labels,
         plddt_labels = plddt_labels,
@@ -440,9 +482,9 @@ def test_alphafold3():
     sampled_atom_pos = alphafold3(
         num_sample_steps = 16,
         atom_inputs = atom_inputs,
-        residue_atom_lens = atom_lens,
-        atompair_feats = atompair_feats,
-        additional_residue_feats = additional_residue_feats,
+        molecule_atom_lens = molecule_atom_lens,
+        atompair_inputs = atompair_inputs,
+        additional_molecule_feats = additional_molecule_feats,
         msa = msa,
         templates = template_feats,
         template_mask = template_mask,
@@ -452,15 +494,15 @@ def test_alphafold3():
 
 def test_alphafold3_without_msa_and_templates():
     seq_len = 16
-    atom_seq_len = seq_len * 27
+    molecule_atom_lens = torch.randint(1, 3, (2, seq_len))
+    atom_seq_len = molecule_atom_lens.sum(dim = -1).amax()
 
     atom_inputs = torch.randn(2, atom_seq_len, 77)
-    atom_lens = torch.randint(0, 27, (2, seq_len))
-    atompair_feats = torch.randn(2, atom_seq_len, atom_seq_len, 16)
-    additional_residue_feats = torch.randn(2, seq_len, 10)
+    atompair_inputs = torch.randn(2, atom_seq_len, atom_seq_len, 5)
+    additional_molecule_feats = torch.randn(2, seq_len, 10)
 
     atom_pos = torch.randn(2, atom_seq_len, 3)
-    residue_atom_indices = torch.randint(0, 27, (2, seq_len))
+    molecule_atom_indices = molecule_atom_lens - 1
 
     distance_labels = torch.randint(0, 38, (2, seq_len, seq_len))
     pae_labels = torch.randint(0, 64, (2, seq_len, seq_len))
@@ -494,11 +536,11 @@ def test_alphafold3_without_msa_and_templates():
     loss, breakdown = alphafold3(
         num_recycling_steps = 2,
         atom_inputs = atom_inputs,
-        residue_atom_lens = atom_lens,
-        atompair_feats = atompair_feats,
-        additional_residue_feats = additional_residue_feats,
+        molecule_atom_lens = molecule_atom_lens,
+        atompair_inputs = atompair_inputs,
+        additional_molecule_feats = additional_molecule_feats,
         atom_pos = atom_pos,
-        residue_atom_indices = residue_atom_indices,
+        molecule_atom_indices = molecule_atom_indices,
         distance_labels = distance_labels,
         pae_labels = pae_labels,
         pde_labels = pde_labels,
@@ -508,89 +550,3 @@ def test_alphafold3_without_msa_and_templates():
     )
 
     loss.backward()
-
-def test_alphafold3_with_packed_atom_repr():
-    seq_len = 16
-    residue_atom_lens = torch.randint(1, 3, (2, seq_len))
-
-    atom_seq_len = residue_atom_lens.sum(dim = -1).amax()
-
-    token_bond = torch.randint(0, 2, (2, seq_len, seq_len)).bool()
-
-    atom_inputs = torch.randn(2, atom_seq_len, 77)
-
-    atompair_feats = torch.randn(2, atom_seq_len, atom_seq_len, 16)
-    additional_residue_feats = torch.randn(2, seq_len, 10)
-
-    template_feats = torch.randn(2, 2, seq_len, seq_len, 44)
-    template_mask = torch.ones((2, 2)).bool()
-
-    msa = torch.randn(2, 7, seq_len, 64)
-    msa_mask = torch.ones((2, 7)).bool()
-
-    atom_pos = torch.randn(2, atom_seq_len, 3)
-    residue_atom_indices = torch.randint(0, 2, (2, seq_len))
-
-    pae_labels = torch.randint(0, 64, (2, seq_len, seq_len))
-    pde_labels = torch.randint(0, 64, (2, seq_len, seq_len))
-    plddt_labels = torch.randint(0, 50, (2, seq_len))
-    resolved_labels = torch.randint(0, 2, (2, seq_len))
-
-    alphafold3 = Alphafold3(
-        dim_atom_inputs = 77,
-        dim_template_feats = 44,
-        num_dist_bins = 38,
-        packed_atom_repr = True,
-        confidence_head_kwargs = dict(
-            pairformer_depth = 1
-        ),
-        template_embedder_kwargs = dict(
-            pairformer_stack_depth = 1
-        ),
-        msa_module_kwargs = dict(
-            depth = 1
-        ),
-        pairformer_stack = dict(
-            depth = 2
-        ),
-        diffusion_module_kwargs = dict(
-            atom_encoder_depth = 1,
-            token_transformer_depth = 1,
-            atom_decoder_depth = 1,
-        ),
-    )
-
-    loss, breakdown = alphafold3(
-        num_recycling_steps = 2,
-        atom_inputs = atom_inputs,
-        residue_atom_lens = residue_atom_lens,
-        atompair_feats = atompair_feats,
-        additional_residue_feats = additional_residue_feats,
-        token_bond = token_bond,
-        msa = msa,
-        msa_mask = msa_mask,
-        templates = template_feats,
-        template_mask = template_mask,
-        atom_pos = atom_pos,
-        residue_atom_indices = residue_atom_indices,
-        pae_labels = pae_labels,
-        pde_labels = pde_labels,
-        plddt_labels = plddt_labels,
-        resolved_labels = resolved_labels,
-        return_loss_breakdown = True
-    )
-
-    loss.backward()
-
-    sampled_atom_pos = alphafold3(
-        num_sample_steps = 16,
-        atom_inputs = atom_inputs,
-        residue_atom_lens = residue_atom_lens,
-        atompair_feats = atompair_feats,
-        additional_residue_feats = additional_residue_feats,
-        msa = msa,
-        templates = template_feats,
-        template_mask = template_mask,
-    )
-
-    assert sampled_atom_pos.ndim == 3
